@@ -1,11 +1,12 @@
 import datetime
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
-from typing import (Any, Callable, List, Optional)
+from typing import (Any, Union, List, Optional)
 
 from fedot.core.chains.chain import Chain
 from fedot.core.data.data import InputData
 from fedot.core.log import Log, default_log
+from fedot.core.repository.quality_metrics_repository import (MetricsEnum)
 
 
 @dataclass
@@ -13,21 +14,23 @@ class ComposerRequirements:
     """
     This dataclass is for defining the requirements of composition process
 
-    :param primary: List of model types (str) for Primary Nodes
-    :param secondary: List of model types (str) for Secondary Nodes
-    :param max_lead_time: max time in minutes available for composition process
-    :param max_depth: max depth of the result chain
-    :param max_arity: maximal number of parent for node
-    :param min_arity: minimal number of parent for node
-    :param add_single_model_chains: allow to have chain with only one node
+    :attribute primary: List of operation types (str) for Primary Nodes
+    :attribute secondary: List of operation types (str) for Secondary Nodes
+    :attribute max_lead_time: max time in minutes available for composition process
+    :attribute max_depth: max depth of the result chain
+    :attribute max_chain_fit_time: time constraint for operation fitting (minutes)
+    :attribute max_arity: maximal number of parent for node
+    :attribute min_arity: minimal number of parent for node
+    :attribute allow_single_operations: allow to have chain with only one node
     """
     primary: List[str]
     secondary: List[str]
     max_lead_time: Optional[datetime.timedelta] = datetime.timedelta(minutes=5)
+    max_chain_fit_time: Optional[datetime.timedelta] = None
     max_depth: int = 3
     max_arity: int = 2
     min_arity: int = 2
-    add_single_model_chains: bool = True
+    allow_single_operations: bool = False
 
     def __post_init__(self):
         if self.max_depth < 0:
@@ -40,7 +43,7 @@ class ComposerRequirements:
 
 class Composer(ABC):
     """
-    Base class used for receiving composite models via optimization
+    Base class used for receiving composite operations via optimization
     :param metrics: metrics used to define the quality of found solution
     :param composer_requirements: requirements for composition process
     :param optimiser_parameters: parameters used by optimization process (i.e. GPComposerRequirements)
@@ -48,7 +51,7 @@ class Composer(ABC):
     :param log: optional parameter for log oject
     """
 
-    def __init__(self, metrics: Optional[Callable], composer_requirements: ComposerRequirements,
+    def __init__(self, metrics: Union[List[MetricsEnum], MetricsEnum], composer_requirements: ComposerRequirements,
                  optimiser_parameters: Any = None, initial_chain: Optional[Chain] = None,
                  log: Log = None):
         self.metrics = metrics
